package pwc.taxtech.atms.controller;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import pwc.taxtech.atms.constant.enums.EnumServiceType;
import pwc.taxtech.atms.dpo.ReportDto;
import pwc.taxtech.atms.dto.OperationResultDto;
import pwc.taxtech.atms.service.impl.CitReportServiceImpl;
import pwc.taxtech.atms.vat.service.impl.ReportServiceImpl;

import java.util.List;
import java.util.Optional;

/**
 * @author zhikai.z.wei
 * @description CIT报表生成（数据处理）
 */
@RestController
@RequestMapping(value = "api/v1/citReport")
public class CitReportController {
    @Autowired
    ReportServiceImpl reportService;

    @Autowired
    CitReportServiceImpl citReportService;

    /**
     * 获取CIT所有要生成的报表模板相关信息，
     * 注意CIT不存在月份期间,默认为0
     * @param projectId
     * @param serviceType
     * @return
     */
    @RequestMapping(value = "citTemplate/{projectId}/{serviceType}", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public OperationResultDto<List<ReportDto>> getCitTemplate(@PathVariable String projectId, @PathVariable int serviceType) {
        return citReportService.getReportTemplate(projectId, EnumServiceType.getEnumByCode(serviceType),0);
    }

    /**
     * 生成CIT所有的报表
     * @param projectId
     * @param generator
     * @param mergeManual
     * @return
     */
    @RequestMapping(value = "generateByTotal/{projectId}/{mergeManual}", method = RequestMethod.POST,
            produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public ResponseEntity generateAllData(@PathVariable String projectId,
                                          @RequestParam Optional<String> generator, @PathVariable Boolean mergeManual) {
        return ResponseEntity.ok(citReportService.generateCitData(projectId, EnumServiceType.CIT, mergeManual,0,null, generator));
    }

    @RequestMapping(value = "filterTemplate/{projectId}/{serviceType}/{period}", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public OperationResultDto<List<ReportDto>> getFilterTemplate(@PathVariable String projectId, @PathVariable int serviceType, @PathVariable Integer period) {
        return citReportService.getFilterReportTemplate(projectId, EnumServiceType.getEnumByCode(serviceType), period);
    }

}