package pwc.taxtech.atms.controller;

import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import pwc.taxtech.atms.common.PageResultVo;
import pwc.taxtech.atms.common.util.DateUtils;
import pwc.taxtech.atms.dto.TaxDocumentDto;
import pwc.taxtech.atms.entity.TaxDocument;
import pwc.taxtech.atms.service.impl.TaxDocumentServiceImpl;
import pwc.taxtech.atms.thirdparty.ExcelUtil;

import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.OutputStream;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;

@Controller
@RequestMapping("/api/v1/taxDoc")
public class TaxDocumentController {

    @Autowired
    private TaxDocumentServiceImpl taxDocumentService;

    @PostMapping("selectList")
    @ResponseBody
    public PageResultVo<TaxDocument> selectTaxDocumentList(@RequestBody TaxDocumentDto taxDocumentDto) {
        Page<TaxDocument> page = PageHelper.startPage(taxDocumentDto.getCurrentPage(), taxDocumentDto.getPageSize());
        taxDocumentService.selectTaxDocumentList(taxDocumentDto);
        PageInfo<TaxDocument> taxDocumentPageInfo = page.toPageInfo();
        List<TaxDocument> list = taxDocumentPageInfo.getList();
        return PageResultVo.getPageResultVo(taxDocumentPageInfo, list);
    }

    @PostMapping("/queryWhetherData")
    @ResponseBody
    public boolean queryWhetherData(@RequestBody TaxDocument taxDocument) {
        return taxDocumentService.queryWhetherData(taxDocument);
    }

    @PostMapping("add")
    @ResponseBody
    public boolean addTaxDocument(TaxDocument taxDocument,
                                  @RequestParam("file") MultipartFile file) {
        return taxDocumentService.addTaxDocumentList(file,taxDocument);
    }

    @PostMapping("delete")
    @ResponseBody
    public boolean deleteTaxDocument(@RequestBody TaxDocument taxDocument) {
        return taxDocumentService.deleteTaxDocument(taxDocument.getId());
    }

    @PostMapping("/batchDelete")
    @ResponseBody
    public boolean batchDelete(@RequestBody TaxDocumentDto taxDocumentDto) {
        return taxDocumentService.batchDelete(taxDocumentDto.getIds());
    }

    @PostMapping("edit")
    @ResponseBody
    public boolean editTaxDocument(@RequestBody TaxDocument taxDocument) {
        return taxDocumentService.editFilesType(taxDocument);
    }

    @RequestMapping("exportExcel")
    @ResponseBody
    public void exportExcelFile(HttpServletResponse response, @RequestBody TaxDocumentDto taxDocumentDto) {
        try {
            Map<String, String> headers = new HashMap<String, String>();
            headers.put("id", "id");
            headers.put("file_attr", "档案属性");
            headers.put("file_type", "档案类型");
            headers.put("file_name", "档案名称");
            headers.put("business_line", "业务线");
            headers.put("company_name", "公司名称");
            headers.put("tax_type", "税种");
            headers.put("file_time", "文件日期");
            headers.put("effective_time", "有效日期");
            headers.put("creator", "创建人");
            headers.put("create_time", "创建时间");
            headers.put("upload_time", "上传日期");
            headers.put("storage_area", "实物存放地点");
            headers.put("keeper", "保管人");
            headers.put("remark", "档案备注");
            headers.put("file_position_url", "文件存储的位置");
            List<TaxDocument> TaxDocument = taxDocumentService.selectTaxDocumentList(taxDocumentDto);
            response.setContentType("multipart/form-data");
            response.setHeader("Content-Disposition", "attachment;fileName=" + new String("".getBytes("GB2312"), "ISO-8859-1"));
            OutputStream ouputStream = response.getOutputStream();
            ExcelUtil.exportExcel(headers, TaxDocument, ouputStream);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * 文件上传接口 createByZhangzezheng
     *
     * @param picture 图片文件
     * @param modual  模块名
     * @return
     */
    @RequestMapping("upload")
    @ResponseBody
    public String upload(@RequestPart("file") MultipartFile picture, @RequestParam(required = false) String modual) {
        return getUploadUrl(picture, modual);
    }

    /**
     * 生成上传url
     *
     * @param file
     * @param modual
     * @return
     */
    private String getUploadUrl(MultipartFile file, String modual) {
        String fileName = file.getOriginalFilename();
        String pictureName = UUID.randomUUID().toString() + fileName.substring(fileName.lastIndexOf("."));
        String dir = DateUtils.getStringDateShort();
        String typePath = "";
        try {
            String fileSavePath = File.separator + "images";
            if (StringUtils.isBlank(modual)) {
                modual = "default";
            }
            if (StringUtils.isNotBlank(modual)) {
                typePath = modual + File.separator + dir;
            }
            File basePath = new File(fileSavePath + File.separator + typePath);
            if (!basePath.exists()) {
                basePath.mkdirs();
            }
            file.transferTo(new File(fileSavePath + File.separator + typePath + File.separator + pictureName));
        } catch (Exception e) {
            e.printStackTrace();
        }
        return "images" + File.separator + typePath + File.separator + pictureName;
    }


}
