package pwc.taxtech.atms.service.impl;

import com.github.pagehelper.PageInfo;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;
import pwc.taxtech.atms.common.AuthUserHelper;
import pwc.taxtech.atms.constant.enums.FileUploadEnum;
import pwc.taxtech.atms.dao.TaxDocumentMapper;
import pwc.taxtech.atms.dto.TaxDocumentDto;
import pwc.taxtech.atms.dto.didiFileUpload.DidiFileIUploadParam;
import pwc.taxtech.atms.dto.didiFileUpload.DidiFileUploadDetailResult;
import pwc.taxtech.atms.entity.OperationLogTaxDocument;
import pwc.taxtech.atms.entity.TaxDocument;
import pwc.taxtech.atms.entity.TaxDocumentExample;
import pwc.taxtech.atms.vat.entity.FileUpload;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 查询
 */
@Slf4j
@Service
public class TaxDocumentServiceImpl {

    @Resource
    private TaxDocumentMapper taxDocumentMapper;

    @Autowired
    private AuthUserHelper authUserHelper;

    @Autowired
    private OperationLogTaxDocServiceImpl operationLogTaxDocService;


    @Autowired
    DidiFileUploadService didiFileUploadService;

    public List<TaxDocument> selectTaxDocumentList(TaxDocumentDto taxDocumentDto) {
        List<TaxDocument> dataList = taxDocumentMapper.selectByExample(getExample(taxDocumentDto));
        DidiFileIUploadParam fileParam = new DidiFileIUploadParam();
        fileParam.setUuids(dataList.stream()
                .map(o -> o.getFileUploadId()).collect(Collectors.toList()));
        PageInfo<DidiFileUploadDetailResult> uploadDetail = didiFileUploadService.queryPage(fileParam);
        Map<String,String> urlMap = null;
        if(CollectionUtils.isNotEmpty(uploadDetail.getList())){
            urlMap = uploadDetail.getList().stream().collect(Collectors.toMap(DidiFileUploadDetailResult::getUid, didiFileUploadDetailResult -> didiFileUploadDetailResult.getViewHttpUrl()));
        }
        if(urlMap!=null){
            for(TaxDocument data:dataList){
                data.setFilePositionUrl(urlMap.get(data.getFileUploadId()));
            }
        }


        return dataList;
    }


    /**
     * 根据条件查询
     *
     * @param taxDocumentDto
     * @return
     */
    private TaxDocumentExample getExample(TaxDocumentDto taxDocumentDto) {
        TaxDocumentExample example = new TaxDocumentExample();
        TaxDocumentExample.Criteria criteria = example.createCriteria();
        //档案属性 fileAttr
        if (StringUtils.isNotBlank(taxDocumentDto.getFileAttr())) {
            criteria.andFileAttrEqualTo(taxDocumentDto.getFileAttr());
        }
        //档案类型 fileType
        if (StringUtils.isNotBlank(taxDocumentDto.getFileType())) {
            criteria.andFileTypeEqualTo(taxDocumentDto.getFileType());
        }
        //文件生效日期 fileTime
        if (null != taxDocumentDto.getFileBeginTime() && null != taxDocumentDto.getFileEndTTime()) {
            criteria.andFileTimeBetween(taxDocumentDto.getFileBeginTime(), taxDocumentDto.getFileEndTTime());
        }
        //所属时间 ownTime
        if (null != taxDocumentDto.getOwnBeginTime() && null != taxDocumentDto.getOwnEndTime()) {
            criteria.andOwnTimeBetween(taxDocumentDto.getOwnBeginTime(), taxDocumentDto.getOwnEndTime());
        }
        //档案名称 fileName
        if (StringUtils.isNotBlank(taxDocumentDto.getFileName())) {
            criteria.andFileNameLike("%"+taxDocumentDto.getFileName()+"%");
        }
        //业务线 businessLine
        if (StringUtils.isNotBlank(taxDocumentDto.getBusinessLine())) {
            criteria.andBusinessLineEqualTo(taxDocumentDto.getBusinessLine());
        }
        //公司名称 companyName
        if (StringUtils.isNotBlank(taxDocumentDto.getCompanyName())) {
            criteria.andCompanyNameEqualTo(taxDocumentDto.getCompanyName());
        }
        //到期日 effectiveTime
        if (null != taxDocumentDto.getEffectiveBeginTime() && null != taxDocumentDto.getEffectiveEndTime()) {
            criteria.andEffectiveTimeBetween(taxDocumentDto.getEffectiveBeginTime(), taxDocumentDto.getEffectiveEndTime());
        }
        //税种 taxType
        if (StringUtils.isNotBlank(taxDocumentDto.getTaxType())) {
            criteria.andTaxTypeEqualTo(taxDocumentDto.getTaxType());
        }
        //实物索引号 physicalIndexNumber
        if (StringUtils.isNotBlank(taxDocumentDto.getPhysicalIndexNumber())) {
            criteria.andPhysicalIndexNumberEqualTo(taxDocumentDto.getPhysicalIndexNumber());
        }
        //实物存放地点 storageArea
        if (StringUtils.isNotBlank(taxDocumentDto.getStorageArea())) {
            criteria.andStorageAreaEqualTo(taxDocumentDto.getStorageArea());
        }
        //实物保管人 keeper
        if (StringUtils.isNotBlank(taxDocumentDto.getKeeper())) {
            criteria.andKeeperEqualTo(taxDocumentDto.getKeeper());
        }
        //审批状态 auditStatus
        if (null != taxDocumentDto.getAuditStatus()) {
            criteria.andAuditStatusEqualTo(taxDocumentDto.getAuditStatus());
        }
        //上传日期 uploadTime
        if (null != taxDocumentDto.getUploadBeginTime() && null != taxDocumentDto.getUploadEndTime()) {
            criteria.andUploadTimeBetween(taxDocumentDto.getUploadBeginTime(), taxDocumentDto.getUploadEndTime());
        }
        //创建人 creator
        if (StringUtils.isNotBlank(taxDocumentDto.getCreator())) {
            criteria.andCreatorEqualTo(taxDocumentDto.getCreator());
        }
        return example;
    }

    @Transactional
    public synchronized boolean addTaxDocumentList(MultipartFile file, TaxDocument taxDocument) {
        try {
            //上传文件
            FileUpload fileUpload = didiFileUploadService.uploadFile(file,file.getOriginalFilename(), FileUploadEnum.BizSource.RECORD_UPLOAD.name());
            taxDocument.setFileUploadId(fileUpload.getUid());
            taxDocument.setFilePositionUrl(fileUpload.getViewHttpUrl());
            //设置创建人 创建时间信息 设置年份区分
            taxDocument.setCreateTime(new Date());
            taxDocument.setCreator(authUserHelper.getCurrentAuditor().get());
            taxDocument.setCreatorId(authUserHelper.getCurrentUserId());
            taxDocument.setUploadTime(new Date());
            taxDocument.setYearRedundancy(Calendar.getInstance().get(Calendar.YEAR));
            int num = taxDocumentMapper.insert(taxDocument);
            if (num > 0) {
                OperationLogTaxDocument actionEntity = buildOperationLogTaxDocument();
                actionEntity.setOperationAction("新增");
                actionEntity.setUpdateState(taxDocument.toString());
                boolean result = operationLogTaxDocService.addTaxDocumentList(actionEntity);
                if (result) {
                    return true;
                } else {
                    throw new RuntimeException("TaxDocumentServiceImpl addTaxDocument log error");
                }
            } else {
                return false;
            }
        } catch (Exception e) {
            log.error("TaxDocumentServiceImpl addTaxDocument error : " + e.getMessage());
            return false;
        }
    }

    public boolean deleteTaxDocument(Long id) {
        try {
//            log.warn("删除的内容:" + taxDocumentMapper.selectByPrimaryKey(id).toString());
//            int num = taxDocumentMapper.deleteByPrimaryKey(id);
            int num = taxDocumentMapper.updateEnableToF(id);
            if (num > 0) {
                OperationLogTaxDocument actionEntity = buildOperationLogTaxDocument();
                actionEntity.setOperationAction("删除");
                boolean result = operationLogTaxDocService.addTaxDocumentList(actionEntity);
                if (result) {
                    return true;
                } else {
                    throw new RuntimeException("TaxDocumentServiceImpl deleteTaxDocument log error");
                }
            } else {
                return false;
            }
        } catch (Exception e) {
            log.error("TaxDocumentServiceImpl deleteTaxDocument error : " + e.getMessage());
            return false;
        }
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @Transactional
    public boolean batchDelete(List<Long> ids) {
        try {
            if ( null == ids || ids.size() < 1) {
                return false;
            }
            for (Long id : ids) {
                taxDocumentMapper.deleteByPrimaryKey(id);
            }
            return true;
        } catch (Exception e) {
            log.error("TaxDocumentServiceImpl batchDelete error : " + e.getMessage());
            return false;
        }
    }

    public boolean editFilesType(TaxDocument taxDocument) {
        try {
            taxDocument.setUpdateTime(new Date());
            int num = taxDocumentMapper.updateByPrimaryKey(taxDocument);
            if (num > 0) {
                OperationLogTaxDocument actionEntity = buildOperationLogTaxDocument();
                actionEntity.setOperationAction("修改");
                //设置更新值
                actionEntity.setOriginalState(taxDocumentMapper.selectByPrimaryKey(taxDocument.getId()).toString());
                actionEntity.setUpdateState(taxDocument.toString());
                boolean result = operationLogTaxDocService.addTaxDocumentList(actionEntity);
                if (result) {
                    return true;
                } else {
                    throw new RuntimeException("TaxDocumentServiceImpl updateTaxDocument log error");
                }
            } else {
                return false;
            }
        } catch (Exception e) {
            log.error("TaxDocumentServiceImpl updateTaxDocument error : " + e.getMessage());
            return false;
        }
    }

    /**
     * 构建日志对象
     *
     * @return
     */
    private OperationLogTaxDocument buildOperationLogTaxDocument() {
        OperationLogTaxDocument actionEntity = new OperationLogTaxDocument();
        actionEntity.setId("TaxDocument:" + UUID.randomUUID().toString());
        actionEntity.setOperationContent("主页");
        actionEntity.setModuleName("档案管理");
        actionEntity.setOperationObject("税务档案");
        actionEntity.setOperationUser(authUserHelper.getCurrentAuditor().get());
        actionEntity.setIp(authUserHelper.getClientIp());
        actionEntity.setCreateTime(new Date());
        return actionEntity;
    }

    /**
     * 根据档案属性 档案类型 公司名称 档案名称 查询是否已存在数据
     * 不存在则返回 true
     * 已存在则返回 false
     *
     * @param taxDocument
     * @return
     */
    public boolean queryWhetherData(TaxDocument taxDocument) {
        try {
            //简单参数校验
       /* if (StringUtils.isAnyBlank(taxDocumentDto.getFileAttr(), taxDocumentDto.getFileName(),
                taxDocumentDto.getFileType(), taxDocumentDto.getCompanyName())){
            return false;
        }*/
            TaxDocumentExample taxDocumentExample = new TaxDocumentExample();
            TaxDocumentExample.Criteria criteria = taxDocumentExample.createCriteria();
            criteria.andFileAttrEqualTo(taxDocument.getFileAttr());
            criteria.andFileTypeEqualTo(taxDocument.getFileType());
            criteria.andFileNameEqualTo(taxDocument.getFileName());
            criteria.andCompanyNameEqualTo(taxDocument.getCompanyName());
            criteria.andOwnTimeEqualTo(taxDocument.getOwnTime());
            List<TaxDocument> taxDocuments = taxDocumentMapper.selectByExample(taxDocumentExample);
            if (null != taxDocuments && taxDocuments.size() > 0) {
                return false;
            }
            return true;
        } catch (Exception e) {
            log.error("TaxDocumentServiceImpl queryWhetherData error :" + e.getMessage());
            return false;
        }
    }
}
